<?php
/**
*
* @package antibot100500
* @copyright (c) 2012,2014 Nekstati http://phpbbguru.net & c61 c61@yandex.ru http://c61.su
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/
namespace c61\antibot100500\event;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
* Event listener
*/
class listener implements EventSubscriberInterface
{	
	/** @var \phpbb\auth\auth */
	protected $auth;
	
	/** @var \phpbb\config\config */
	protected $config;
	
	/** @var \phpbb\template\template */
	protected $template;
	
	/** @var \phpbb\user */
	protected $user;
	
	/** @var \phpbb\db\driver\driver_interface */
	protected $db;
	
	/** @var string */
	protected $phpbb_root_path;
	protected $php_ext;
	
	/**
	* Constructor
	* 
	* @param \phpbb\auth\auth $auth
	* @param \phpbb\config\config $config
	* @param \phpbb\template\template $template
	* @param \phpbb\user $user
	* @param \phpbb\db\driver\driver_interface $db
	* @param string $phpbb_root_path Root path
	* @param string $php_ext
	*/
	public function __construct(\phpbb\auth\auth $auth, \phpbb\config\config $config, \phpbb\template\template $template, \phpbb\user $user, \phpbb\db\driver\driver_interface $db, $phpbb_root_path, $php_ext)
	{
		$this->auth = $auth;
		$this->config = $config;
		$this->template = $template;
		$this->user = $user;
		$this->db = $db;
		$this->phpbb_root_path = $phpbb_root_path;
		$this->php_ext = $php_ext;
	}
	
	/**
	* Assign functions defined in this class to event listeners in the core
	*
	* @return array
	* @static
	* @access public
	*/
	static public function getSubscribedEvents()
	{
		return array(
			/**
			* Event to load language files and modify user data on every page
			*
			* @event core.user_setup
			* @var	array	user_data			Array with user's data row
			* @var	string	user_lang_name		Basename of the user's langauge
			* @var	string	user_date_format	User's date/time format
			* @var	string	user_timezone		User's timezone, should be one of
			*							http://www.php.net/manual/en/timezones.php
			* @var	mixed	lang_set			String or array of language files
			* @var	array	lang_set_ext		Array containing entries of format
			* 					array(
			* 						'ext_name' => (string) [extension name],
			* 						'lang_set' => (string|array) [language files],
			* 					)
			* 					For performance reasons, only load translations
			* 					that are absolutely needed globally using this
			* 					event. Use local events otherwise.
			* @var	mixed	style_id			Style we are going to display
			* @since 3.1.0-a1
			*/
			'core.user_setup'			=> 'lang_antibot100500',

			/**
			* Execute code and/or overwrite _common_ template variables after they have been assigned.
			*
			* @event core.page_header_after
			* @var	string	page_title			Page title
			* @var	bool	display_online_list		Do we display online users list
			* @var	string	item				Restrict online users to a certain
			*									session item, e.g. forum for
			*									session_forum_id
			* @var	int		item_id				Restrict online users to item id
			* @var	array		http_headers			HTTP headers that should be set by phpbb
			*
			* @since 3.1.0-b3
			*/
			'core.page_header_after'	=> 'add_scripts_antibot100500',

			/**
			* Perform additional actions on creation of the form token
			*
			* @event core.add_form_key
			* @var	string	form_name			The form name
			* @var	int		now					Current time timestamp
			* @var	string	s_fields			Generated hidden fields
			* @var	string	token				Form token
			* @var	string	token_sid			User session ID
			*
			* @since 3.1.0-RC3
			*/
			'core.add_form_key'			=> array('form_key_antibot100500',-10001),

			/**
			* Execute code and/or modify output before displaying the template.
			*
			* @event core.page_footer_after
			* @var	bool display_template	Whether or not to display the template
			* @var	bool exit_handler		Whether or not to run the exit_handler()
			*
			* @since 3.1.0-RC5
			*/
			'core.page_footer_after'	=> 'log_antibot100500',

			/**
			* This event allows you to define errors before the post action is performed
			*
			* @event core.posting_modify_submission_errors
			* @var	array	post_data	Array with post data
			* @var	array	poll		Array with poll data from post (must be used instead of the post_data equivalent)
			* @var	string	mode		What action to take if the form is submitted
			*				post|reply|quote|edit|delete|bump|smilies|popup
			* @var	string	page_title	Title of the mode page
			* @var	int	post_id		ID of the post
			* @var	int	topic_id	ID of the topic
			* @var	int	forum_id	ID of the forum
			* @var	bool	submit		Whether or not the form has been submitted
			* @var	array	error		Any error strings; a non-empty array aborts form submission.
			*				NOTE: Should be actual language strings, NOT language keys.
			* @since 3.1.0-RC5
			* @change 3.1.5-RC1 Added poll array to the event
			*/
			'core.posting_modify_submission_errors'	=> array('log_posting_debug_antibot100500',10001),
		);
	}
	
	// ON/OFF
	private $onoff_antibot = false;
	// ON/OFF for registered users
	private $onoff_registered = false;
	// Options
	private $onoff_encode = false;
	private $onoff_behavior = false;
	// Logs
	private $onoff_log_users = false;
	private $onoff_pdlog_users = false;
	// Correct message
	private $onoff_correct_message = false;
	private $add_message = '';

	// Mobile browser detected
	private $mobile_browser = false;

	/** Encode with key(xor) **/
	private	$key = 85;	// DO NOT CHANGE VALUE !!!

	//
	// Settings
	//
	private function settings() {

		// NOT for adm ot install
		if ( defined('ADMIN_START') || defined('IN_INSTALL') ) return FALSE;

		// ON/OFF
		if ( isset($this->config['antibot100500_onoff_antibot']) ) {
			$this->onoff_antibot = $this->config['antibot100500_onoff_antibot'] ? true : false;
		}
		// Switched OFF ?
		if ( !$this->onoff_antibot ) return FALSE;
		
		// Log
		if ( isset($this->config['antibot100500_onoff_log_users']) ) {
			$this->onoff_log_users = $this->config['antibot100500_onoff_log_users'] ? true : false;
		}

		// PDLog
		if ( isset($this->config['antibot100500_onoff_pdlog_users']) ) {
			$this->onoff_pdlog_users = $this->config['antibot100500_onoff_pdlog_users'] ? true : false;
		}
		// Correct Message
		if ( isset($this->config['antibot100500_onoff_correct_message']) ) {
			$this->onoff_correct_message = $this->config['antibot100500_onoff_correct_message'] ? true : false;
		}

		// ON/OFF for registered users
		if ( isset($this->config['antibot100500_onoff_registered']) ) {
			$this->onoff_registered = $this->config['antibot100500_onoff_registered'] ? true : false;
		}

		// Options for paranoids
		if ( isset($this->config['antibot100500_onoff_encode']) ) {
			$this->onoff_encode = $this->config['antibot100500_onoff_encode'] ? true : false;
		}
		if ( isset($this->config['antibot100500_onoff_behavior']) ) {
			$this->onoff_behavior = $this->config['antibot100500_onoff_behavior'] ? true : false;
		}

		return TRUE;
	}

	//
	// Encode
	//
	private function encode($in, $key) {

		$out = '';
		for ( $i = 0; $i < strlen($in); $i++, $out .= $i != strlen($in) ? ';' : '' ) {
			$out .= strval(ord(substr($in,$i,1)) ^ $key);
		}
		return $out;
	}

	/**
	* Events
	*/

	//
	// Lang
	//
	public function lang_antibot100500($event)
	{
		$lang_set_ext = $event['lang_set_ext'];
		$lang_set_ext[] = array(
			'ext_name' => 'c61/antibot100500',
			'lang_set' => 'antibot100500',
		);
		$event['lang_set_ext'] = $lang_set_ext;
	}

	//
	// Add scripts
	//
	public function add_scripts_antibot100500($event) {

		if ( !$this->settings() ) return;

		// Add scripts
		// Script for keyboard and mouse events processing
		if ( $this->onoff_behavior ) {
			$this->template->assign_vars(array(
				'ANTIBOT100500_FORM_TOKEN_EVENT'	=> $this->onoff_encode ? 'enc' : 'min',
			));
		}
		// Decoder script
		if ( $this->onoff_encode ) {
			$this->template->assign_vars(array(
				'ANTIBOT100500_DECODER_SCRIPT'	=> '
var func_form_tokens = function(s) {
	var _0xe3ad=["\x37\x28\x34\x20\x32\x3D\x39\x2E\x62\x28\x27\x3B\x27\x29\x2C\x69\x3D\x30\x2C\x33\x3D\x27\x27\x3B\x69\x3C\x32\x2E\x38\x3B\x69\x2B\x2B\x29\x33\x2B\x3D\x63\x2E\x61\x28\x32\x5B\x69\x5D\x5E\x64\x29\x3B\x34\x20\x31\x3D\x35\x2E\x6B\x28\x27\x65\x27\x29\x3B\x31\x2E\x6C\x3D\x27\x36\x2F\x6A\x27\x3B\x31\x2E\x36\x3D\x33\x3B\x35\x2E\x68\x2E\x66\x5B\x30\x5D\x2E\x67\x28\x31\x29\x3B","\x7C","\x73\x70\x6C\x69\x74","\x7C\x6A\x73\x7C\x78\x7C\x7A\x7C\x76\x61\x72\x7C\x64\x6F\x63\x75\x6D\x65\x6E\x74\x7C\x74\x65\x78\x74\x7C\x66\x6F\x72\x7C\x6C\x65\x6E\x67\x74\x68\x7C\x73\x7C\x66\x72\x6F\x6D\x43\x68\x61\x72\x43\x6F\x64\x65\x7C\x73\x70\x6C\x69\x74\x7C\x53\x74\x72\x69\x6E\x67\x7C\x38\x35\x7C\x53\x43\x52\x49\x50\x54\x7C\x63\x68\x69\x6C\x64\x72\x65\x6E\x7C\x61\x70\x70\x65\x6E\x64\x43\x68\x69\x6C\x64\x7C\x64\x6F\x63\x75\x6D\x65\x6E\x74\x45\x6C\x65\x6D\x65\x6E\x74\x7C\x7C\x6A\x61\x76\x61\x73\x63\x72\x69\x70\x74\x7C\x63\x72\x65\x61\x74\x65\x45\x6C\x65\x6D\x65\x6E\x74\x7C\x74\x79\x70\x65","\x72\x65\x70\x6C\x61\x63\x65","","\x5C\x77\x2B","\x5C\x62","\x67"];eval(function(_0x5fa5x1,_0x5fa5x2,_0x5fa5x3,_0x5fa5x4,_0x5fa5x5,_0x5fa5x6){_0x5fa5x5=function(_0x5fa5x3){return _0x5fa5x3.toString(36)};if(!_0xe3ad[5][_0xe3ad[4]](/^/,String)){while(_0x5fa5x3--){_0x5fa5x6[_0x5fa5x3.toString(_0x5fa5x2)]=_0x5fa5x4[_0x5fa5x3]||_0x5fa5x3.toString(_0x5fa5x2)};_0x5fa5x4=[function(_0x5fa5x5){return _0x5fa5x6[_0x5fa5x5]}];_0x5fa5x5=function(){return _0xe3ad[6]};_0x5fa5x3=1;};while(_0x5fa5x3--){if(_0x5fa5x4[_0x5fa5x3]){_0x5fa5x1=_0x5fa5x1[_0xe3ad[4]]( new RegExp(_0xe3ad[7]+_0x5fa5x5(_0x5fa5x3)+_0xe3ad[7],_0xe3ad[8]),_0x5fa5x4[_0x5fa5x3])}};return _0x5fa5x1;}(_0xe3ad[0],22,22,_0xe3ad[3][_0xe3ad[2]](_0xe3ad[1]),0,{}));
}
',
			));
		}
	}

	//
	// Change form_token value
	//
	public function form_key_antibot100500($event) {

		if ( !$this->settings() ) return;

		// All vars OK ?
		if ( !isset($event['s_fields']) ) return;
		if ( !isset($event['token']) ) return;
		if ( !isset($event['now']) ) return;
		if ( !isset($event['form_name']) ) return;

		// OFF for registered users and registered user ?
		if ( !$this->onoff_registered && !empty($this->user->data['is_registered']) && $this->user->data['is_registered'] ) {

			if ( $this->onoff_pdlog_users || $this->onoff_correct_message ) {
				// Log swithed ON
				$add_s_fields = build_hidden_fields(array(
					'form_token_name'       => $event['form_name'],
					'form_token_session_id' => $this->user->session_id,
					'form_token_token_hash'	=> dechex(crc32($event['token'])),
					'form_token_now_hash'	=> dechex(crc32(strval($event['now']))),
				));

				// Modify input form_token
				$event['s_fields'] = $add_s_fields . $event['s_fields'];
			}

			return;
		}

		// Add "onsubmit" event (behavios analisis) for mobile browsers
		// NOTE: NOT compatible with Motorola Internet Browser
//
// see: http://detectmobilebrowsers.com/
//
		if ( $this->onoff_behavior ) {

			global $request;

			$useragent = $request->server('HTTP_USER_AGENT');
			if ( !empty($useragent) ) {
	       		if(preg_match('/(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows ce|xda|xiino/i',$useragent)||preg_match('/1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\-(n|u)|c55\/|capi|ccwa|cdm\-|cell|chtm|cldc|cmd\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\-s|devi|dica|dmob|do(c|p)o|ds(12|\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\-|_)|g1 u|g560|gene|gf\-5|g\-mo|go(\.w|od)|gr(ad|un)|haie|hcit|hd\-(m|p|t)|hei\-|hi(pt|ta)|hp( i|ip)|hs\-c|ht(c(\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\-(20|go|ma)|i230|iac( |\-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\/)|klon|kpt |kwc\-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|\-[a-w])|libw|lynx|m1\-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m\-cr|me(rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\-2|po(ck|rt|se)|prox|psio|pt\-g|qa\-a|qc(07|12|21|32|60|\-[2-7]|i\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\-|oo|p\-)|sdk\/|se(c(\-|0|1)|47|mc|nd|ri)|sgh\-|shar|sie(\-|m)|sk\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\-|v\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\-|tdg\-|tel(i|m)|tim\-|t\-mo|to(pl|sh)|ts(70|m\-|m3|m5)|tx\-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas\-|your|zeto|zte\-/i',substr($useragent,0,4))) {
					$this->mobile_browser = true;
				}
			}
		}

		// form_token framing
		$subprev = rand(28,32);
		$subsucc = 60 - $subprev;
		$prev = substr(md5(dechex(rand(1,2147483647))),0,$subprev);
		$succ = substr(md5($this->user->session_id),0,$subsucc);
		$token = $prev . $event['token'] . $succ;
		$marker = $this->onoff_behavior ? '' : sprintf('%u',crc32($token));

		// Unframing script
		$script_src = '
	var dec_' . $marker . '_form_tokens = function() {
		for ( var i=0; i<name_' . $marker . '_form_tokens.length; i++ ) {
			name_' . $marker . '_form_tokens[i].value = name_' . $marker . '_form_tokens[i].value.replace(/' . $prev . '|' . $succ . '/g, \'\');
		}
	}
';

		// Start scripts & Input element
		$code = '
<script type="text/javascript">// <![CDATA[

	var name_' . $marker . '_form_tokens = document.getElementsByName("form_token");
';

		//
		// see minificator:
		//	http://jscompress.com/
		//
		// see obfuscators:
		//	1) http://www.daftlogic.com/projects-online-javascript-obfuscator.htm
		//	2) http://javascriptobfuscator.com/Javascript-Obfuscator.aspx
		// deobfuscator http://jsbeautifier.org/ can't deobfuscate double obfuscation
		//

		// Script for keyboard and mouse events processing
		// see http://javascript.ru/tutorial/events/crossbrowser
		// ... already added in styles/template/overall_header_head_append.html

		// Original decoder script
/*
		for ( var x = s.split(';'), i=0, z=''; i<x.length; i++) z += String.fromCharCode(x[i]^85);
		var js = document.createElement('SCRIPT');
		js.type = 'text/javascript';
		js.text = z;
		document.documentElement.children[0].appendChild(js);
*/
		// Double obfuscated decoder script
		// ... already added in styles/template/overall_header_head_append.html
		
		// Events handling and decoding
		if ( $this->onoff_behavior ) $code .= '
	var handle_' . $marker . '_form_tokens = function(event) {
		for ( var i=0; i<name_' . $marker . '_form_tokens.length; i++ ) {
			form_token_Event.remove(name_' . $marker . '_form_tokens[i].form,\'keydown\',handle_' . $marker . '_form_tokens);
			form_token_Event.remove(name_' . $marker . '_form_tokens[i].form,\'mousedown\',handle_' . $marker . '_form_tokens);'
. ($this->mobile_browser ?'
			form_token_Event.remove(name_' . $marker . '_form_tokens[i].form,\'submit\',handle_' . $marker . '_form_tokens);' : '').'
		}
		dec_' . $marker . '_form_tokens();
	}
';

		// Start decoder
		if ( $this->onoff_encode ) $code .= '
	func_form_tokens(String("' . $this->encode($script_src,$this->key) . '"));
';
		// ... or original script
		else $code .= $script_src;

		// If no behavioral analysis
		if ( !$this->onoff_behavior ) $code .= '
	dec_' . $marker . '_form_tokens();
';
		// Add events
		if ( $this->onoff_behavior ) $code .= '
	for ( var i=0; i<name_' . $marker . '_form_tokens.length; i++ ) {
		form_token_Event.add(name_' . $marker . '_form_tokens[i].form,\'keydown\',handle_' . $marker . '_form_tokens);
		form_token_Event.add(name_' . $marker . '_form_tokens[i].form,\'mousedown\',handle_' . $marker . '_form_tokens);'
.($this->mobile_browser ? '
		form_token_Event.add(name_' . $marker . '_form_tokens[i].form,\'submit\',handle_' . $marker . '_form_tokens);' : '').'
	}
';
		// End scripts
		$code .= '
// ]]></script>
';

		// Log swithed ON ?
		$add_s_fields = '';
		if ( $this->onoff_pdlog_users || $this->onoff_correct_message ) {
			$add_s_fields = build_hidden_fields(array(
				'form_token_name'       => $event['form_name'],
				'form_token_session_id' => $this->user->session_id,
				'form_token_token_hash'	=> dechex(crc32($event['token'])),
				'form_token_now_hash'	=> dechex(crc32(strval($event['now']))),
			));
		}

		// Modify input form_token
		$event['s_fields'] = $add_s_fields . preg_replace('#(<input type="hidden" name="form_token" value=")([0-9a-f]+)(" />)#', '${1}' . $prev . '${2}' . $succ . '$3' . $code, $event['s_fields']);
		$event['token'] = $token;

	}

	//
	// Logging
	//
	public function log_antibot100500($event) {

		if ( !$this->settings() ) return;

		if ( !$this->onoff_log_users ) return;

		// rootref context

		global $phpbb_container, $phpbb_log;

		$context = $phpbb_container->get('template_context');
		$rootref = &$context->get_root_ref();
		
		// Form invalid detected...
		if ( !empty($rootref['ERROR'])
		  && (strpos($this->user->lang['FORM_INVALID'],$rootref['ERROR']) !== false)
		   ) {

			$log_operation = 'LOG_ANTIBOT100500_FORM_INVALID_ON';

			$user_email = '';
			$user_name = '<br />user » ';
			if ( !empty($rootref['S_USER_LOGGED_IN']) && $rootref['S_USER_LOGGED_IN'] ) {
				// ...possible, posting: user logged in
				$user_name .= !empty($rootref['S_USERNAME']) ? $rootref['S_USERNAME'] : $this->user->lang['GUEST'];
				// OFF for registered users and registered user ?
				if ( !$this->onoff_registered && !empty($this->user->data['is_registered']) && $this->user->data['is_registered'] ) {
					$log_operation = 'LOG_ANTIBOT100500_FORM_INVALID_OFF_USER';
				}
			} else {
				// ...logging in, registering, etc.
				$user_name .= !empty($rootref['USERNAME']) ? $rootref['USERNAME'] : $this->user->lang['GUEST'];
				if ( empty($user_name) ) $user_name = !empty($rotref['S_USERNAME']) ? $rotref['S_USERNAME'] : '';
				$user_email = !empty($rootref['EMAIL']) ? $rootref['EMAIL'] : '';
			}

			$user_name .= '<br />script » '.$rootref['SCRIPT_NAME'];
			$user_name .= '<br />useragent » '.$rootref['S_USER_BROWSER'];
			if ( !empty($user_email) ) $user_name .= '<br />email » '.$user_email;

			$user_ip = (empty($this->user->ip)) ? '' : $this->user->ip;

			$additional_data = array();
			$additional_data['reportee_id'] = $this->user->data['user_id'];
			$additional_data = array_merge($additional_data, array($user_name));

			$phpbb_log->add('user', $this->user->data['user_id'], $user_ip, $log_operation, time(), $additional_data);

		}
	}

	//
	// Logging on posting debug
	//
	public function log_posting_debug_antibot100500($event) {

		if ( !$this->settings() ) return;

		global $request, $phpbb_log;

		// Form invalid detected...
		$error = $event['error'];
		if ( !empty($error)
		  && (in_array($this->user->lang['FORM_INVALID'],$error) !== false)
		   ) {
			$log_operation = 'LOG_ANTIBOT100500_FORM_INVALID_ON';
			$this->add_message = '';

			$user_name = '<strong> (posting/core.posting_modify_submission_errors)</strong><br />user » ';
			if ( $this->user->data['user_id'] != ANONYMOUS ) {
				// ...possible, posting: user logged in
				$user_name .= $this->user->data['username'];
				// OFF for registered users and registered user ?
				if ( !$this->onoff_registered && !empty($this->user->data['is_registered']) && $this->user->data['is_registered'] ) {
					$log_operation = 'LOG_ANTIBOT100500_FORM_INVALID_OFF_USER';
				}
			} else {
				// ...logging in, registering, etc.
				$user_name .= $this->user->data['username'];
			}

			$user_name .= '<br />page » '.$this->user->page['page_name'];
			$user_name .= '<br />useragent » '.(isset($this->user->data['session_browser']) ? $this->user->data['session_browser'] : $this->user->lang['UNKNOWN_BROWSER']);

			$user_name .= '<br /><strong>posting status:</strong>';

			$user_name .= '<br />mode » '.$event['mode'];
			$user_name .= '<br />submit » '.($event['submit'] ? 'true' : 'false');

			if ( $request->is_set_post('form_token_name') ) {
				$form_token_name = $request->variable('form_token_name', '');
				$user_name .= '<br />form_token_name » ' . $form_token_name;
				if ( $form_token_name != 'posting' ) {
					$this->add_message .= $this->user->lang['ANTIBOT100500_FORM_TOKEN_NAME'];
				}
			}

			$user_name .= '<br /><strong>session:</strong>';

			$user_name .= '<br />time_now » '.$this->user->time_now;
			$user_name .= '<br />session_time » '.$this->user->data['session_time'];

			$user_name .= '<br /><strong>request' . ($request->is_ajax() ? ' (AJAX)' : '') . ':</strong>';

			$user_name .= '<br />creation_time » ';
			if ( $request->is_set_post('creation_time') ) {
				$creation_time = abs($request->variable('creation_time', 0));
				$user_name .= $creation_time;
				if ( $request->is_set_post('form_token_now_hash') ) {
					$form_token_now_hash = $request->variable('form_token_now_hash', '');
					if ( $form_token_now_hash != dechex(crc32(strval($creation_time))) ) {
						$user_name .= ' <strong>NOT</strong> equal to original';
						$this->add_message .= $this->user->lang['ANTIBOT100500_FORM_TOKEN_NOW_CHANGED'];
					}
				}
			} else {
				$user_name .= 'undefined';
				$this->add_message .= $this->user->lang['ANTIBOT100500_FORM_CREATION_TIME_NOT_FOUND'];
			}

			$user_name .= '<br />form_token » ';
			if ( $request->is_set_post('form_token') ) {
				$form_token = $request->variable('form_token', '');
				$user_name .= $form_token;
				if ( $request->is_set_post('form_token_token_hash') ) {
					$form_token_token_hash = $request->variable('form_token_token_hash', '');
					if ( $form_token_token_hash != dechex(crc32($form_token)) ) {
						$user_name .= ' <strong>NOT</strong> equal to original';
						$this->add_message .= $this->user->lang['ANTIBOT100500_FORM_TOKEN_TOKEN_CHANGED'];
					}
				}
			} else {
				$user_name .= 'undefined';
				$this->add_message .= $this->user->lang['ANTIBOT100500_FORM_TOKEN_NOT_FOUND'];
			}

			if ( $request->is_set_post('form_token_session_id') ) {
				$form_token_session_id = $request->variable('form_token_session_id', '');
				$user_name .= '<br />form_token_session_id » ' . ((($form_token_session_id == $this->user->session_id)) ? 'not changed' : ('<strong>changed</strong>, was ' . $form_token_session_id . ', now ' . $this->user->session_id) );
				if ( $form_token_session_id != $this->user->session_id ) {
					$this->add_message .= $this->user->lang['ANTIBOT100500_SESSION_TIME_ELAPSED'];
				}
			}

			if ( isset($creation_time) && isset($form_token) ) {

				$timespan = ($this->config['form_token_lifetime'] == -1) ? -1 : max(30, $this->config['form_token_lifetime']);
				$diff = time() - $creation_time;

				$user_name .= '<br /><strong>check form key:</strong>';

				$user_name .= '<br />timespan » '.$timespan;
				$user_name .= '<br />diff time » '.$diff;

				if (defined('DEBUG_TEST') || $diff && ($diff <= $timespan || $timespan === -1)) {
					$token_sid = ($this->user->data['user_id'] == ANONYMOUS && !empty($this->config['form_token_sid_guests'])) ? $this->user->session_id : '';
					$key = sha1($creation_time . $this->user->data['user_form_salt'] . 'posting' . $token_sid);

					$user_name .= '<br />sid » ' . (empty($token_sid) ? 'empty' : $token_sid);
					
					$user_name .= '<br />key ';
					if ( false !== ($i = strpos($form_token,$key)) ) {
						if ( $i ) {
							$user_name .= '(found in token, probably, JavaScript is <strong>OFF</strong>)';
							$this->add_message .= $this->user->lang['ANTIBOT100500_JAVASCRIPT_OFF'];
						} else if ( (string) $form_key != (string) $key ) {
							$user_name .= '(found in token, <strong>NOT</strong> equal to original)';
						} else {
							$user_name .= '(found in token)';
						}
					} else {
						$user_name .= '(<strong>NOT</strong> found in token)';
						$this->add_message .= $this->user->lang['ANTIBOT100500_TOKEN_NOT_MATCH_KEY'];
					}						
					$user_name .= ' » ' . $key;
				} else {
					$user_name .= '<br /><strong>form lifetime elapsed</strong>';
					$this->add_message .= $this->user->lang['ANTIBOT100500_FORM_TIME_ELAPSED'];
				}
			}

			// Correct
			if ( !empty($this->add_message) ) {
				$this->add_message = rtrim(str_replace('.',', ',$this->add_message),', ') . '.';
			}

			// Log swithed ON ?
			if ( $this->onoff_pdlog_users ) {

				if ( !empty($this->add_message) ) {
					$user_name .= '<br /><strong>' . $this->user->lang['ANTIBOT100500_ADD_MESSAGE'] . $this->add_message . '</strong>';
				}

				$user_ip = (empty($this->user->ip)) ? '' : $this->user->ip;

				$additional_data = array();
				$additional_data['reportee_id'] = $this->user->data['user_id'];
				$additional_data = array_merge($additional_data, array($user_name));

				$phpbb_log->add('user', $this->user->data['user_id'], $user_ip, $log_operation, time(), $additional_data);

			}

			// Correct message ?
			if ( $this->onoff_correct_message && !empty($this->add_message) ) {
			
				$br = '<br />';
				if ( $request->is_ajax() ) {
					$br .= '<br />';
					$this->add_message .=  $br . $this->user->lang['ANTIBOT100500_ADD_MESSAGE_AJAX'];
				}

				$event['error'] = str_replace($this->user->lang['FORM_INVALID'],$this->user->lang['FORM_INVALID'] . $br . $this->user->lang['ANTIBOT100500_ADD_MESSAGE'] . $this->add_message,$error);

			}

		}
	}
}
